@@ -1,33 +1,36 @@
 import os
+import sys
 
 import click
 
+from semantic_release import ci_checks
+
 from .history import (evaluate_version_bump, get_current_version, get_new_version,
                       get_previous_version, set_new_version)
 from .history.logs import CHANGELOG_SECTIONS, generate_changelog, markdown_changelog
 from .hvcs import check_build_status, check_token, post_changelog
 from .pypi import upload_to_pypi
 from .settings import config
-from .vcs_helpers import (commit_new_version, get_current_head_hash, get_repository_owner_and_name,
-                          push_new_version, tag_new_version)
+from .vcs_helpers import (checkout, commit_new_version, get_current_head_hash,
+                          get_repository_owner_and_name, push_new_version, tag_new_version)
+
+_common_options = [
+    click.option('--major', 'force_level', flag_value='major', help='Force major version.'),
+    click.option('--minor', 'force_level', flag_value='minor', help='Force minor version.'),
+    click.option('--patch', 'force_level', flag_value='patch', help='Force patch version.'),
+    click.option('--post', is_flag=True, help='Post changelog.'),
+    click.option('--noop', is_flag=True,
+                 help='No-operations mode, finds the new version number without changing it.')
+]
 
 
-@click.command()
-@click.argument('command')
-@click.option('--major', 'force_level', flag_value='major', help='Force major version.')
-@click.option('--minor', 'force_level', flag_value='minor', help='Force minor version.')
-@click.option('--patch', 'force_level', flag_value='patch', help='Force patch version.')
-@click.option('--post', is_flag=True, help='Post changelog.')
-@click.option('--noop', is_flag=True,
-              help='No-operations mode, finds the new version number without changing it.')
-def main(command, **kwargs):
+def common_options(func):
     """
-    Routes to the correct cli function. Looks up the command
-    in the global scope and calls the function.
-    :param command: String with the name of the function to call
-    :param kwargs: All click options.
+    Decorator that adds all the options in _common_options
     """
-    globals()[command](**kwargs)
+    for option in reversed(_common_options):
+        func = option(func)
+    return func
 
 
 def version(**kwargs):
@@ -61,8 +64,9 @@ def version(**kwargs):
             return False
         click.echo(click.style('The build was a success, continuing the release', 'green'))
 
-    set_new_version(new_version)
-    commit_new_version(new_version)
+    if config.get('semantic_release', 'version_source') == 'commit':
+        set_new_version(new_version)
+        commit_new_version(new_version)
     tag_new_version(new_version)
     click.echo('Bumping with a {0} version to {1}.'.format(level_bump, new_version))
     return True
@@ -73,9 +77,10 @@ def changelog(**kwargs):
     Generates the changelog since the last release.
     """
     current_version = get_current_version()
-    log = generate_changelog(get_previous_version(current_version), current_version)
+    log = generate_changelog(
+        get_previous_version(current_version), current_version)
     for section in CHANGELOG_SECTIONS:
-        if len(log[section]) == 0:
+        if not log[section]:
             continue
 
         click.echo(section.capitalize())
@@ -95,7 +100,8 @@ def changelog(**kwargs):
                 markdown_changelog(current_version, log, header=False)
             )
         else:
-            click.echo(click.style('Missing token: cannot post changelog', 'red'), err=True)
+            click.echo(
+                click.style('Missing token: cannot post changelog', 'red'), err=True)
 
 
 def publish(**kwargs):
@@ -108,6 +114,9 @@ def publish(**kwargs):
     new_version = get_new_version(current_version, level_bump)
     owner, name = get_repository_owner_and_name()
 
+    ci_checks.check('master')
+    checkout('master')
+
     if version(**kwargs):
         push_new_version(
             gh_token=os.environ.get('GH_TOKEN'),
@@ -116,7 +125,10 @@ def publish(**kwargs):
         )
 
         if config.getboolean('semantic_release', 'upload_to_pypi'):
-            upload_to_pypi()
+            upload_to_pypi(
+                username=os.environ.get('PYPI_USERNAME'),
+                password=os.environ.get('PYPI_PASSWORD'),
+            )
 
         if check_token():
             click.echo('Updating changelog')
@@ -128,12 +140,51 @@ def publish(**kwargs):
                 markdown_changelog(new_version, log, header=False)
             )
         else:
-            click.echo(click.style('Missing token: cannot post changelog', 'red'), err=True)
+            click.echo(
+                click.style('Missing token: cannot post changelog', 'red'), err=True)
 
         click.echo(click.style('New release published', 'green'))
     else:
         click.echo('Version failed, no release will be published.', err=True)
 
 
+#
+# Making the CLI commands.
+# We have a level of indirection to the logical commands
+# so we can successfully mock them during testing
+#
+
+@click.group()
+@common_options
+def main(**kwargs):
+    pass
+
+
+@main.command(name='publish', help=publish.__doc__)
+@common_options
+def cmd_publish(**kwargs):
+    return publish(**kwargs)
+
+
+@main.command(name='changelog', help=changelog.__doc__)
+@common_options
+def cmd_changelog(**kwargs):
+    return changelog(**kwargs)
+
+
+@main.command(name='version', help=version.__doc__)
+@common_options
+def cmd_version(**kwargs):
+    return version(**kwargs)
+
+
 if __name__ == '__main__':
-    main()
+    #
+    # Allow options to come BEFORE commands,
+    # we simply sort them behind the command instead.
+    #
+    # This will have to be removed if there are ever global options
+    # that are not valid for a subcommand.
+    #
+    args = sorted(sys.argv[1:], key=lambda x: 1 if x.startswith('--') else -1)
+    main(args=args)
